@@ -9,22 +9,22 @@ mod texture_cube_map;
 #[doc(inline)]
 pub use texture_cube_map::*;
 
-mod color_target_texture2d;
-#[doc(inline)]
-pub use color_target_texture2d::*;
-
 mod depth_target_texture2d;
 #[doc(inline)]
 pub use depth_target_texture2d::*;
 
-mod color_target_texture2d_array;
+mod texture2d_array;
 #[doc(inline)]
-pub use color_target_texture2d_array::*;
+pub use texture2d_array::*;
 
 mod depth_target_texture2d_array;
 #[doc(inline)]
 pub use depth_target_texture2d_array::*;
 
+mod depth_target_texture_cube_map;
+#[doc(inline)]
+pub use depth_target_texture_cube_map::*;
+
 ///
 /// Possible modes of interpolation which determines the texture output between texture pixels.
 ///
@@ -53,8 +53,10 @@ pub trait TextureDataType:
 {
 }
 impl TextureDataType for u8 {}
-impl TextureDataType for f32 {}
+impl TextureDataType for u16 {}
 impl TextureDataType for u32 {}
+impl TextureDataType for f16 {}
+impl TextureDataType for f32 {}
 
 ///
 /// Possible formats for pixels in a texture.
@@ -81,7 +83,7 @@ impl Format {
 }
 
 ///
-/// A CPU-side version of a texture, for example [Texture2D].
+/// A CPU-side version of a [Texture2D].
 /// Can be constructed manually or loaded via [Loader](crate::Loader).
 ///
 #[allow(missing_docs)]
@@ -89,14 +91,14 @@ pub struct CPUTexture<T: TextureDataType> {
     pub data: Vec<T>,
     pub width: u32,
     pub height: u32,
-    pub depth: u32,
     pub format: Format,
     pub min_filter: Interpolation,
     pub mag_filter: Interpolation,
+    /// Specifies whether mipmaps should be created for this texture and what type of interpolation to use between the two closest mipmaps.
+    /// Note, however, that the mipmaps only will be created if the width and height of the texture are power of two.
     pub mip_map_filter: Option<Interpolation>,
     pub wrap_s: Wrapping,
     pub wrap_t: Wrapping,
-    pub wrap_r: Wrapping,
 }
 
 impl<T: TextureDataType> CPUTexture<T> {
@@ -133,14 +135,12 @@ impl<T: TextureDataType> Default for CPUTexture<T> {
             data: [T::default(), T::default(), T::default(), T::default()].into(),
             width: 1,
             height: 1,
-            depth: 1,
             format: Format::RGBA,
             min_filter: Interpolation::Linear,
             mag_filter: Interpolation::Linear,
             mip_map_filter: Some(Interpolation::Linear),
             wrap_s: Wrapping::Repeat,
             wrap_t: Wrapping::Repeat,
-            wrap_r: Wrapping::Repeat,
         }
     }
 }
@@ -152,7 +152,76 @@ impl<T: TextureDataType> std::fmt::Debug for CPUTexture<T> {
             .field("data length", &self.data.len())
             .field("width", &self.width)
             .field("height", &self.height)
-            .field("depth", &self.depth)
+            .field("min_filter", &self.min_filter)
+            .field("mag_filter", &self.mag_filter)
+            .field("mip_map_filter", &self.mip_map_filter)
+            .field("wrap_s", &self.wrap_s)
+            .field("wrap_t", &self.wrap_t)
+            .finish()
+    }
+}
+
+///
+/// A CPU-side version of a [TextureCubeMap]. All 6 images must have the same dimensions.
+/// Can be constructed manually or loaded via [Loader](crate::Loader).
+///
+#[allow(missing_docs)]
+pub struct CPUTextureCube<T: TextureDataType> {
+    /// The pixel data for the right image
+    pub right_data: Vec<T>,
+    /// The pixel data for the left image
+    pub left_data: Vec<T>,
+    /// The pixel data for the top image
+    pub top_data: Vec<T>,
+    /// The pixel data for the bottom image
+    pub bottom_data: Vec<T>,
+    /// The pixel data for the front image
+    pub front_data: Vec<T>,
+    /// The pixel data for the back image
+    pub back_data: Vec<T>,
+    /// The width of each of the 6 images
+    pub width: u32,
+    /// The height of each of the 6 images
+    pub height: u32,
+    pub format: Format,
+    pub min_filter: Interpolation,
+    pub mag_filter: Interpolation,
+    /// Specifies whether mipmaps should be created for this texture and what type of interpolation to use between the two closest mipmaps.
+    /// Note, however, that the mipmaps only will be created if the width and height of the texture are power of two.
+    pub mip_map_filter: Option<Interpolation>,
+    pub wrap_s: Wrapping,
+    pub wrap_t: Wrapping,
+    pub wrap_r: Wrapping,
+}
+
+impl<T: TextureDataType> Default for CPUTextureCube<T> {
+    fn default() -> Self {
+        Self {
+            right_data: vec![],
+            left_data: vec![],
+            top_data: vec![],
+            bottom_data: vec![],
+            front_data: vec![],
+            back_data: vec![],
+            width: 1,
+            height: 1,
+            format: Format::RGBA,
+            min_filter: Interpolation::Linear,
+            mag_filter: Interpolation::Linear,
+            mip_map_filter: Some(Interpolation::Linear),
+            wrap_s: Wrapping::Repeat,
+            wrap_t: Wrapping::Repeat,
+            wrap_r: Wrapping::Repeat,
+        }
+    }
+}
+
+impl<T: TextureDataType> std::fmt::Debug for CPUTextureCube<T> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("CPUTexture")
+            .field("format", &self.format)
+            .field("width", &self.width)
+            .field("height", &self.height)
             .field("min_filter", &self.min_filter)
             .field("mag_filter", &self.mag_filter)
             .field("mip_map_filter", &self.mip_map_filter)
@@ -179,6 +248,7 @@ pub(in crate::core) mod internal {
         );
         fn read(context: &Context, viewport: Viewport, format: Format, pixels: &mut [Self]);
         fn is_max(value: Self) -> bool;
+        fn bits_per_channel() -> u8;
     }
 
     impl TextureDataTypeExtension for u8 {
@@ -227,7 +297,120 @@ pub(in crate::core) mod internal {
         fn is_max(value: Self) -> bool {
             value == 255u8
         }
+
+        fn bits_per_channel() -> u8 {
+            8
+        }
     }
+
+    impl TextureDataTypeExtension for u16 {
+        fn internal_format(format: Format) -> ThreeDResult<u32> {
+            Ok(match format {
+                Format::R => crate::context::consts::R16UI,
+                Format::RG => crate::context::consts::RG16UI,
+                Format::RGB => crate::context::consts::RGB16UI,
+                Format::RGBA => crate::context::consts::RGBA16UI,
+            })
+        }
+
+        fn fill(
+            context: &Context,
+            target: u32,
+            width: u32,
+            height: u32,
+            format: Format,
+            data: &[Self],
+        ) {
+            context.tex_sub_image_2d_with_u16_data(
+                target,
+                0,
+                0,
+                0,
+                width,
+                height,
+                format_from(format),
+                DataType::UnsignedShort,
+                data,
+            );
+        }
+
+        fn read(context: &Context, viewport: Viewport, format: Format, pixels: &mut [Self]) {
+            context.read_pixels_with_u16_data(
+                viewport.x as u32,
+                viewport.y as u32,
+                viewport.width as u32,
+                viewport.height as u32,
+                format_from(format),
+                DataType::UnsignedShort,
+                pixels,
+            );
+        }
+
+        fn is_max(value: Self) -> bool {
+            value == std::u16::MAX
+        }
+
+        fn bits_per_channel() -> u8 {
+            16
+        }
+    }
+
+    impl TextureDataTypeExtension for f16 {
+        fn internal_format(format: Format) -> ThreeDResult<u32> {
+            Ok(match format {
+                Format::R => crate::context::consts::R16F,
+                Format::RG => crate::context::consts::RG16F,
+                Format::RGB => crate::context::consts::RGB16F,
+                Format::RGBA => crate::context::consts::RGBA16F,
+            })
+        }
+
+        fn fill(
+            context: &Context,
+            target: u32,
+            width: u32,
+            height: u32,
+            format: Format,
+            data: &[Self],
+        ) {
+            context.tex_sub_image_2d_with_u16_data(
+                target,
+                0,
+                0,
+                0,
+                width,
+                height,
+                format_from(format),
+                DataType::HalfFloat,
+                &data.iter().map(|v| v.to_bits()).collect::<Vec<_>>(),
+            );
+        }
+
+        fn read(context: &Context, viewport: Viewport, format: Format, pixels: &mut [Self]) {
+            let mut pixels_temp = vec![0u16; pixels.len()];
+            context.read_pixels_with_u16_data(
+                viewport.x as u32,
+                viewport.y as u32,
+                viewport.width as u32,
+                viewport.height as u32,
+                format_from(format),
+                DataType::HalfFloat,
+                &mut pixels_temp,
+            );
+            for i in 0..pixels.len() {
+                pixels[i] = f16::from_bits(pixels_temp[i]);
+            }
+        }
+
+        fn is_max(value: Self) -> bool {
+            value > f16::from_f32(0.99)
+        }
+
+        fn bits_per_channel() -> u8 {
+            16
+        }
+    }
+
     impl TextureDataTypeExtension for f32 {
         fn internal_format(format: Format) -> ThreeDResult<u32> {
             Ok(match format {
@@ -274,6 +457,10 @@ pub(in crate::core) mod internal {
         fn is_max(value: Self) -> bool {
             value > 0.99
         }
+
+        fn bits_per_channel() -> u8 {
+            32
+        }
     }
 
     impl TextureDataTypeExtension for u32 {
@@ -321,6 +508,10 @@ pub(in crate::core) mod internal {
         fn is_max(_value: Self) -> bool {
             true
         }
+
+        fn bits_per_channel() -> u8 {
+            32
+        }
     }
 
     fn format_from(format: Format) -> u32 {
@@ -352,6 +543,42 @@ pub trait Texture {
     fn is_transparent(&self) -> bool;
 }
 
+impl<T: Texture + ?Sized> Texture for &T {
+    fn bind(&self, location: u32) {
+        (*self).bind(location)
+    }
+    fn width(&self) -> u32 {
+        (*self).width()
+    }
+    fn height(&self) -> u32 {
+        (*self).height()
+    }
+    fn format(&self) -> Format {
+        (*self).format()
+    }
+    fn is_transparent(&self) -> bool {
+        (*self).is_transparent()
+    }
+}
+
+impl<T: Texture + ?Sized> Texture for &mut T {
+    fn bind(&self, location: u32) {
+        (**self).bind(location)
+    }
+    fn width(&self) -> u32 {
+        (**self).width()
+    }
+    fn height(&self) -> u32 {
+        (**self).height()
+    }
+    fn format(&self) -> Format {
+        (**self).format()
+    }
+    fn is_transparent(&self) -> bool {
+        (**self).is_transparent()
+    }
+}
+
 ///
 /// A texture array that can be sampled in a fragment shader (see [use_texture_array](crate::core::Program::use_texture_array)).
 ///
@@ -368,6 +595,42 @@ pub trait TextureArray {
     fn format(&self) -> Format;
 }
 
+impl<T: TextureArray + ?Sized> TextureArray for &T {
+    fn bind(&self, location: u32) {
+        (*self).bind(location)
+    }
+    fn width(&self) -> u32 {
+        (*self).width()
+    }
+    fn height(&self) -> u32 {
+        (*self).height()
+    }
+    fn depth(&self) -> u32 {
+        (*self).depth()
+    }
+    fn format(&self) -> Format {
+        (*self).format()
+    }
+}
+
+impl<T: TextureArray + ?Sized> TextureArray for &mut T {
+    fn bind(&self, location: u32) {
+        (**self).bind(location)
+    }
+    fn width(&self) -> u32 {
+        (**self).width()
+    }
+    fn height(&self) -> u32 {
+        (**self).height()
+    }
+    fn depth(&self) -> u32 {
+        (**self).depth()
+    }
+    fn format(&self) -> Format {
+        (**self).format()
+    }
+}
+
 ///
 /// A texture cube that can be sampled in a fragment shader (see [use_texture_cube](crate::core::Program::use_texture_cube)).
 ///
@@ -380,6 +643,44 @@ pub trait TextureCube {
     fn height(&self) -> u32;
     /// The format of this texture.
     fn format(&self) -> Format;
+    /// Whether or not this texture contain hdr information.
+    fn is_hdr(&self) -> bool;
+}
+
+impl<T: TextureCube + ?Sized> TextureCube for &T {
+    fn bind(&self, location: u32) {
+        (*self).bind(location)
+    }
+    fn width(&self) -> u32 {
+        (*self).width()
+    }
+    fn height(&self) -> u32 {
+        (*self).height()
+    }
+    fn format(&self) -> Format {
+        (*self).format()
+    }
+    fn is_hdr(&self) -> bool {
+        (*self).is_hdr()
+    }
+}
+
+impl<T: TextureCube + ?Sized> TextureCube for &mut T {
+    fn bind(&self, location: u32) {
+        (**self).bind(location)
+    }
+    fn width(&self) -> u32 {
+        (**self).width()
+    }
+    fn height(&self) -> u32 {
+        (**self).height()
+    }
+    fn format(&self) -> Format {
+        (**self).format()
+    }
+    fn is_hdr(&self) -> bool {
+        (**self).is_hdr()
+    }
 }
 
 // COMMON TEXTURE FUNCTIONS
@@ -459,13 +760,11 @@ fn calculate_number_of_mip_maps(
     mip_map_filter: Option<Interpolation>,
     width: u32,
     height: u32,
-    depth: u32,
 ) -> u32 {
-    if mip_map_filter.is_some() {
+    if mip_map_filter.is_some() && width == height && height.is_power_of_two() {
         let w = (width as f64).log2().ceil();
         let h = (height as f64).log2().ceil();
-        let d = (depth as f64).log2().ceil();
-        w.max(h).max(d).floor() as u32 + 1
+        w.max(h).floor() as u32 + 1
     } else {
         1
     }
