@@ -1,4 +1,3 @@
-use crate::io::*;
 use std::path::Path;
 
 ///
@@ -10,7 +9,7 @@ impl Saver {
     ///
     /// Save the byte array as a file.
     ///
-    pub fn save_file<P: AsRef<Path>>(path: P, bytes: &[u8]) -> ThreeDResult<()> {
+    pub fn save_file<P: AsRef<Path>>(path: P, bytes: &[u8]) -> crate::ThreeDResult<()> {
         let mut file = std::fs::File::create(path)?;
         use std::io::prelude::*;
         file.write_all(bytes)?;
