@@ -40,20 +40,28 @@ pub enum RendererError {}
 /// Render the objects. Also avoids rendering objects outside the camera frustum and render the objects in the order given by [cmp_render_order].
 /// Must be called in a render target render function, for example in the callback function of [Screen::write].
 ///
-pub fn render_pass(camera: &Camera, objects: &[&dyn Object], lights: &Lights) -> ThreeDResult<()> {
+pub fn render_pass(camera: &Camera, objects: &[impl Object], lights: &Lights) -> ThreeDResult<()> {
     let mut culled_objects = objects
         .iter()
         .filter(|o| camera.in_frustum(&o.aabb()))
         .collect::<Vec<_>>();
-    culled_objects.sort_by(|a, b| cmp_render_order(camera, **a, **b));
+    culled_objects.sort_by(|a, b| cmp_render_order(camera, a, b));
     for object in culled_objects {
         object.render(camera, lights)?;
     }
     Ok(())
 }
 
-use std::cmp::Ordering;
-pub fn cmp_render_order(camera: &Camera, obj0: &dyn Object, obj1: &dyn Object) -> Ordering {
+///
+/// Compare function for sorting objects based on distance from the camera.
+/// The order is opaque objects from nearest to farthest away from the camera,
+/// then transparent objects from farthest away to closest to the camera.
+///
+pub fn cmp_render_order(
+    camera: &Camera,
+    obj0: impl Object,
+    obj1: impl Object,
+) -> std::cmp::Ordering {
     if obj0.is_transparent() == obj1.is_transparent() {
         let distance_a = camera.position().distance2(obj0.aabb().center());
         let distance_b = camera.position().distance2(obj1.aabb().center());
@@ -64,33 +72,13 @@ pub fn cmp_render_order(camera: &Camera, obj0: &dyn Object, obj1: &dyn Object) -
         }
     } else {
         if obj0.is_transparent() {
-            Ordering::Greater
+            std::cmp::Ordering::Greater
         } else {
-            Ordering::Less
+            std::cmp::Ordering::Less
         }
     }
 }
 
-impl crate::core::Camera {
-    ///
-    /// Finds the closest intersection between a ray from this camera in the given pixel coordinate and the given geometries.
-    /// The pixel coordinate must be in physical pixels, where (viewport.x, viewport.y) indicate the top left corner of the viewport
-    /// and (viewport.x + viewport.width, viewport.y + viewport.height) indicate the bottom right corner.
-    /// Returns ```None``` if no geometry was hit before the given maximum depth.
-    ///
-    #[deprecated = "Use pick function instead"]
-    pub fn pick<S: Shadable>(
-        &self,
-        pixel: (f32, f32),
-        max_depth: f32,
-        objects: &[S],
-    ) -> ThreeDResult<Option<Vec3>> {
-        let pos = self.position_at_pixel(pixel);
-        let dir = self.view_direction_at_pixel(pixel);
-        ray_intersect(&self.context, pos, dir, max_depth, objects)
-    }
-}
-
 ///
 /// Finds the closest intersection between a ray from the given camera in the given pixel coordinate and the given geometries.
 /// The pixel coordinate must be in physical pixels, where (viewport.x, viewport.y) indicate the top left corner of the viewport
@@ -142,7 +130,7 @@ pub fn ray_intersect<S: Shadable>(
         0.0,
         max_depth,
     )?;
-    let texture = ColorTargetTexture2D::<f32>::new(
+    let mut texture = Texture2D::<f32>::new_empty(
         context,
         viewport.width,
         viewport.height,
@@ -153,7 +141,7 @@ pub fn ray_intersect<S: Shadable>(
         Wrapping::ClampToEdge,
         Format::RGBA,
     )?;
-    let depth_texture = DepthTargetTexture2D::new(
+    let mut depth_texture = DepthTargetTexture2D::new(
         context,
         viewport.width,
         viewport.height,
@@ -171,20 +159,22 @@ pub fn ray_intersect<S: Shadable>(
         },
         ..Default::default()
     };
-    let render_target = RenderTarget::new(context, &texture, &depth_texture)?;
-    render_target.write(
-        ClearState {
-            red: Some(1.0),
-            depth: Some(1.0),
-            ..ClearState::none()
-        },
-        || {
-            for geometry in geometries {
-                geometry.render_forward(&depth_material, &camera, &Lights::default())?;
-            }
-            Ok(())
-        },
-    )?;
+    {
+        let render_target = RenderTarget::new(context, &mut texture, &mut depth_texture)?;
+        render_target.write(
+            ClearState {
+                red: Some(1.0),
+                depth: Some(1.0),
+                ..ClearState::none()
+            },
+            || {
+                for geometry in geometries {
+                    geometry.render_with_material(&depth_material, &camera, &Lights::default())?;
+                }
+                Ok(())
+            },
+        )?;
+    }
     let depth = texture.read(viewport)?[0];
     Ok(if depth < 1.0 {
         Some(position + direction * depth * max_depth)
